package com.fjhx.form.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.utils.Assert;
import com.fjhx.form.entity.RemitDetailSelectDto;
import com.fjhx.form.entity.RemitDetailVo;
import com.fjhx.form.entity.SubjectBalanceBo;
import com.fjhx.form.entity.SubjectBalanceSelectDto;
import com.fjhx.form.entity.account.po.AccountSubjectsBalance;
import com.fjhx.form.mapper.FinanceReportMapper;
import com.fjhx.form.service.FinanceReportService;
import com.fjhx.form.service.account.AccountSubjectsBalanceService;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class FinanceReportServiceImpl implements FinanceReportService {
    @Resource
    private FinanceReportMapper financeReportMapper;
    @Resource
    private AccountSubjectsBalanceService accountSubjectsBalanceService;

    @Override
    public List<SubjectBalanceBo> subjectBalance(SubjectBalanceSelectDto dto) {
        Assert.notEmpty(dto.getYear(), "年份不能为空！");
        IWrapper<Object> wrapper = IWrapper.getWrapper();


        wrapper.apply("YEAR ( t1.expenses_time ) = {0}", dto.getYear());

        wrapper.keyword(dto.getKeyword(),
                new SqlField("asj.subjects_name"),
                new SqlField("t1.calculate_item_str")
        );


        Map<Long, AccountSubjectsBalance> subjectsBalanceMap = accountSubjectsBalanceService.mapKEntity(AccountSubjectsBalance::getSubjectsId, q -> q
                .apply("YEAR(account_date) = {0} AND MONTH(account_date) = 12", dto.getYear() - 1)
        );


        SubjectBalanceBo totalBo = new SubjectBalanceBo();


        List<SubjectBalanceBo> subjectBalanceBos = financeReportMapper.subjectBalance(wrapper, dto);
        for (SubjectBalanceBo subjectBalanceBo : subjectBalanceBos) {

            //获取期出余额
            BigDecimal yearBeginBalance = BigDecimal.ZERO;
            AccountSubjectsBalance accountSubjectsBalance = subjectsBalanceMap.get(subjectBalanceBo.getSubjectsId());
            if (accountSubjectsBalance != null) {
                yearBeginBalance = accountSubjectsBalance.getBalance();
            }
            subjectBalanceBo.setYearBeginBalance(yearBeginBalance);
            //余额 = 年初余额 + 本期发生额
            subjectBalanceBo.setBalanceAmount(yearBeginBalance.add(subjectBalanceBo.getTradeAmount()));

            totalBo.setSubjectsName("合计");
            totalBo.setYearBeginBalance(ObjectUtil.defaultIfNull(totalBo.getYearBeginBalance(), BigDecimal.ZERO).add(subjectBalanceBo.getYearBeginBalance()));
            totalBo.setMonth1Amount(ObjectUtil.defaultIfNull(totalBo.getMonth1Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth1Amount()));
            totalBo.setMonth2Amount(ObjectUtil.defaultIfNull(totalBo.getMonth2Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth2Amount()));
            totalBo.setMonth3Amount(ObjectUtil.defaultIfNull(totalBo.getMonth3Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth3Amount()));
            totalBo.setMonth4Amount(ObjectUtil.defaultIfNull(totalBo.getMonth4Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth4Amount()));
            totalBo.setMonth5Amount(ObjectUtil.defaultIfNull(totalBo.getMonth5Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth5Amount()));
            totalBo.setMonth6Amount(ObjectUtil.defaultIfNull(totalBo.getMonth6Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth6Amount()));
            totalBo.setMonth7Amount(ObjectUtil.defaultIfNull(totalBo.getMonth7Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth7Amount()));
            totalBo.setMonth8Amount(ObjectUtil.defaultIfNull(totalBo.getMonth8Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth8Amount()));
            totalBo.setMonth9Amount(ObjectUtil.defaultIfNull(totalBo.getMonth9Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth9Amount()));
            totalBo.setMonth10Amount(ObjectUtil.defaultIfNull(totalBo.getMonth10Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth10Amount()));
            totalBo.setMonth11Amount(ObjectUtil.defaultIfNull(totalBo.getMonth11Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth11Amount()));
            totalBo.setMonth12Amount(ObjectUtil.defaultIfNull(totalBo.getMonth12Amount(), BigDecimal.ZERO).add(subjectBalanceBo.getMonth12Amount()));
            totalBo.setTradeAmount(ObjectUtil.defaultIfNull(totalBo.getTradeAmount(), BigDecimal.ZERO).add(subjectBalanceBo.getTradeAmount()));
            totalBo.setBalanceAmount(ObjectUtil.defaultIfNull(totalBo.getBalanceAmount(), BigDecimal.ZERO).add(subjectBalanceBo.getBalanceAmount()));
        }

        subjectBalanceBos.add(totalBo);

        return subjectBalanceBos;
    }

    /**
     * 每月1日0点1分 赋值客户上月科目余额
     */
//    @PostConstruct
    @Scheduled(cron = "0 1 0 1 * ?")
    public void autoSaveReceivableAccountsEnding() {
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        SecurityUtils.setTenantId("000000");

        DateTime lastMonth = DateUtil.offsetMonth(new Date(), -1);


        SubjectBalanceSelectDto dto = new SubjectBalanceSelectDto();
        dto.setYear(DateUtil.year(lastMonth));
        List<SubjectBalanceBo> subjectBalanceBos = subjectBalance(dto);

        for (SubjectBalanceBo receivableAccount : subjectBalanceBos) {
            Long subjectsId = receivableAccount.getSubjectsId();
            int month = DateUtil.month(lastMonth) + 1;


            DateTime parse = DateUtil.parse(StrUtil.format("{}-{}", dto.getYear(), month), "yyyy-MM");
            AccountSubjectsBalance one = accountSubjectsBalanceService.getOne(q -> q
                    .eq(AccountSubjectsBalance::getSubjectsId, subjectsId)
                    .eq(AccountSubjectsBalance::getAccountDate, parse)
            );
            if (ObjectUtil.isEmpty(one)) {
                one = new AccountSubjectsBalance();
                one.setSubjectsId(subjectsId);
                one.setAccountDate(parse);
            }

            JSONObject json = JSONObject.parseObject(JSONObject.toJSONString(receivableAccount));
            one.setBalance(json.getBigDecimal("month" + month + "Amount"));
            accountSubjectsBalanceService.saveOrUpdate(one);
        }

        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.clear();
    }

    @Override
    public Page<RemitDetailVo> getRemitDetailPage(RemitDetailSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();

        wrapper.keyword(dto.getKeyword(),
                new SqlField("t1.contract_codes"),
                new SqlField("t2.subject_name"),
                new SqlField("t1.customerShortName"),
                new SqlField("t2.supplier_names"),
                new SqlField("cc.voucher_no"),
                new SqlField("arw.amount"),
                new SqlField("am.`alias`")
        );

        //交易时间范围过滤
        wrapper.ge("arw.transaction_time", dto.getBeginTime());
        wrapper.le("arw.transaction_time", dto.getEndTime());

        wrapper.orderByDesc("arw.transaction_time");
        Page<RemitDetailVo> page = financeReportMapper.getRemitDetailPage(dto.getPage(), wrapper);
        return page;
    }
}
