package com.fjhx.oa.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.invoice.dto.InvoiceTaxDeductionDto;
import com.fjhx.oa.entity.invoice.po.InvoiceTaxDeduction;
import com.fjhx.oa.service.invoice.InvoiceTaxDeductionService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class InvoiceTaxDeductionFlow extends FlowDelegate {

    @Autowired
    private InvoiceTaxDeductionService invoiceTaxDeductionService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "invoice_tax_deduction_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        InvoiceTaxDeductionDto dto = submitData.toJavaObject(InvoiceTaxDeductionDto.class);
        dto.setFlowId(flowId);

        dto.setCode(codingRuleService.createCode(CodingRuleEnum.INVOICE_TAX_DEDUCTION.getKey(), null));
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        invoiceTaxDeductionService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        invoiceTaxDeductionService.update(q -> q
                .eq(InvoiceTaxDeduction::getId, businessId)
                .set(InvoiceTaxDeduction::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        invoiceTaxDeductionService.update(q -> q
                .eq(InvoiceTaxDeduction::getId, businessId)
                .set(InvoiceTaxDeduction::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        invoiceTaxDeductionService.cancellation(businessId);
    }
}
